@@ -1,28 +1,48 @@
-from typing import Tuple, Optional, Callable, Iterable, Dict, TYPE_CHECKING
+# pylint:disable=import-outside-toplevel
+from __future__ import annotations
+from typing import TYPE_CHECKING
+from collections.abc import Callable, Iterable
 import queue
 import threading
 import time
 import logging
 from collections import defaultdict
+from enum import Enum
+
+import networkx
 
 import claripy
 
-from ..utils.mp import mp_context, Initializer
-from ..knowledge_plugins.cfg import CFGModel
-from ..analyses.cfg import CFGUtils
-from . import Analysis, register_analysis, VariableRecoveryFast, CallingConventionAnalysis
+from angr.utils.graph import GraphUtils
+from angr.simos import SimWindows
+from angr.utils.mp import mp_context, Initializer
+from angr.knowledge_plugins.cfg import CFGModel
+from . import Analysis, register_analysis, VariableRecoveryFast, CallingConventionAnalysis, FactCollector, CFGFast
 
 if TYPE_CHECKING:
-    import networkx
     from angr.calling_conventions import SimCC
     from angr.sim_type import SimTypeFunction
     from angr.knowledge_plugins.variables.variable_manager import VariableManagerInternal
+    from angr.knowledge_plugins.functions.function_manager import Function
+
 
 _l = logging.getLogger(name=__name__)
 
 _mp_context = mp_context()
 
 
+class CallingConventionAnalysisMode(Enum):
+    """
+    The mode of calling convention analysis.
+
+    FAST: Using FactCollector to collect facts, then use facts for calling convention analysis.
+    VARIABLES: Using variables in VariableManager for calling convention analysis.
+    """
+
+    FAST = "fast"
+    VARIABLES = "variables"
+
+
 class CompleteCallingConventionsAnalysis(Analysis):
     """
     Implements full-binary calling convention analysis. During the initial analysis of a binary, you may set
@@ -32,20 +52,21 @@ class CompleteCallingConventionsAnalysis(Analysis):
 
     def __init__(
         self,
+        mode: CallingConventionAnalysisMode = CallingConventionAnalysisMode.FAST,
         recover_variables=False,
         low_priority=False,
         force=False,
-        cfg: Optional[CFGModel] = None,
+        cfg: CFGFast | CFGModel | None = None,
         analyze_callsites: bool = False,
         skip_signature_matched_functions: bool = False,
-        max_function_blocks: Optional[int] = None,
-        max_function_size: Optional[int] = None,
+        max_function_blocks: int | None = None,
+        max_function_size: int | None = None,
         workers: int = 0,
-        cc_callback: Optional[Callable] = None,
-        prioritize_func_addrs: Optional[Iterable[int]] = None,
+        cc_callback: Callable | None = None,
+        prioritize_func_addrs: list[int] | set[int] | None = None,
         skip_other_funcs: bool = False,
         auto_start: bool = True,
-        func_graphs: Optional[Dict[int, "networkx.DiGraph"]] = None,
+        func_graphs: dict[int, networkx.DiGraph] | None = None,
     ):
         """
 
@@ -64,10 +85,11 @@ class CompleteCallingConventionsAnalysis(Analysis):
         :param workers:             Number of multiprocessing workers.
         """
 
+        self.mode = mode
         self._recover_variables = recover_variables
         self._low_priority = low_priority
         self._force = force
-        self._cfg = cfg
+        self._cfg: CFGModel | None = cfg.model if isinstance(cfg, CFGFast) else cfg
         self._analyze_callsites = analyze_callsites
         self._skip_signature_matched_functions = skip_signature_matched_functions
         self._max_function_blocks = max_function_blocks
@@ -78,17 +100,24 @@ class CompleteCallingConventionsAnalysis(Analysis):
         self._skip_other_funcs = skip_other_funcs
         self._auto_start = auto_start
         self._total_funcs = None
-        self._func_graphs = {} if not func_graphs else func_graphs
+        self._func_graphs = func_graphs if func_graphs else {}
+        self.prototype_libnames: set[str] = set()
+
+        # sanity check
+        if self.mode not in {CallingConventionAnalysisMode.FAST, CallingConventionAnalysisMode.VARIABLES}:
+            raise ValueError(f"Invalid calling convention analysis mode {self.mode}.")
 
         self._func_addrs = []  # a list that holds addresses of all functions to be analyzed
         self._results = []
         if workers > 0:
             self._remaining_funcs = _mp_context.Value("i", 0)
-            self._func_queue = _mp_context.Queue()
             self._results = _mp_context.Queue()
+            self._results_lock = _mp_context.Lock()
+            self._func_queue = _mp_context.Queue()
             self._func_queue_lock = _mp_context.Lock()
         else:
             self._remaining_funcs = None  # not needed
+            self._results_lock = None  # not needed
             self._func_queue = None  # not needed
             self._func_queue_lock = threading.Lock()
 
@@ -101,14 +130,27 @@ class CompleteCallingConventionsAnalysis(Analysis):
         Infer calling conventions for all functions in the current project.
         """
 
-        # get an ordering of functions based on the call graph
-        sorted_funcs = CFGUtils.quasi_topological_sort_nodes(self.kb.functions.callgraph)
+        # special case: if both _prioritize_func_addrs and _skip_other_funcs are set, we only need to sort part of
+        # the call graph; even better, if there is only one function set, we don't need to sort the call graph at all!
+        if self._prioritize_func_addrs and self._skip_other_funcs:
+            if len(self._prioritize_func_addrs) == 1:
+                self._func_addrs = list(self._prioritize_func_addrs)
+                self._total_funcs = 1
+                return
+            directed_callgraph = networkx.DiGraph(self.kb.functions.callgraph)
+            directed_callgraph = directed_callgraph.subgraph(self._prioritize_func_addrs)
+        else:
+            # get an ordering of functions based on the call graph
+            # note that the call graph is a multi-digraph. we convert it to a digraph to speed up topological sort
+            directed_callgraph = networkx.DiGraph(self.kb.functions.callgraph)
+        assert isinstance(directed_callgraph, networkx.DiGraph)
+        sorted_funcs = GraphUtils.quasi_topological_sort_nodes(directed_callgraph)
 
         total_funcs = 0
         for func_addr in reversed(sorted_funcs):
             func = self.kb.functions.get_by_addr(func_addr)
             if (func.calling_convention is None or func.prototype is None) or self._force:
-                if func.alignment:
+                if func.is_alignment:
                     # skip all alignments
                     continue
 
@@ -117,7 +159,7 @@ class CompleteCallingConventionsAnalysis(Analysis):
                     continue
 
                 if self._max_function_size is not None:
-                    func_size = sum(block.size for block in func.blocks)
+                    func_size = sum(block.size for block in func.blocks if block.size is not None)
                     if func_size > self._max_function_size:
                         _l.info(
                             "Skipping variable recovery for %r since its size (%d) is greater than the cutoff "
@@ -128,16 +170,15 @@ class CompleteCallingConventionsAnalysis(Analysis):
                         )
                         continue
 
-                if self._max_function_blocks is not None:
-                    if len(func.block_addrs_set) > self._max_function_blocks:
-                        _l.info(
-                            "Skipping variable recovery for %r since its number of blocks (%d) is greater than the "
-                            "cutoff number (%d).",
-                            func,
-                            len(func.block_addrs_set),
-                            self._max_function_blocks,
-                        )
-                        continue
+                if self._max_function_blocks is not None and len(func.block_addrs_set) > self._max_function_blocks:
+                    _l.info(
+                        "Skipping variable recovery for %r since its number of blocks (%d) is greater than the "
+                        "cutoff number (%d).",
+                        func,
+                        len(func.block_addrs_set),
+                        self._max_function_blocks,
+                    )
+                    continue
 
                 # if it's a normal function, we attempt to perform variable recovery
                 self._func_addrs.append(func_addr)
@@ -149,31 +190,40 @@ class CompleteCallingConventionsAnalysis(Analysis):
             self.prioritize_functions(self._prioritize_func_addrs)
         self._prioritize_func_addrs = None  # no longer useful
 
+    def _set_function_prototype(
+        self, func: Function, prototype: SimTypeFunction | None, prototype_libname: str | None
+    ) -> None:
+        if func.prototype is None or func.is_prototype_guessed or self._force:
+            func.is_prototype_guessed = True
+            func.prototype = prototype
+            func.prototype_libname = prototype_libname
+
     def work(self):
         total_funcs = self._total_funcs
+        assert total_funcs is not None
         if self._workers == 0:
-            idx = 0
             self._update_progress(0)
-            for func_addr in self._func_addrs:
-                cc, proto, _ = self._analyze_core(func_addr)
+            for idx, func_addr in enumerate(self._func_addrs):
+                cc, proto, proto_libname, _ = self._analyze_core(func_addr)
 
                 func = self.kb.functions.get_by_addr(func_addr)
                 if cc is not None or proto is not None:
                     func.calling_convention = cc
-                    func.prototype = proto
-                    func.is_prototype_guessed = True
+                    self._set_function_prototype(func, proto, proto_libname)
+                    if proto_libname is not None:
+                        self.prototype_libnames.add(proto_libname)
 
                 if self._cc_callback is not None:
                     self._cc_callback(func_addr)
 
-                idx += 1
-
-                percentage = idx / total_funcs * 100.0
-                self._update_progress(percentage, text=f"{idx}/{total_funcs} - {func.demangled_name}")
+                percentage = (idx + 1) / total_funcs * 100.0
+                self._update_progress(percentage, text=f"{idx + 1}/{total_funcs} - {func.demangled_name}")
                 if self._low_priority:
-                    self._release_gil(idx, 10, 0.000001)
+                    self._release_gil(idx + 1, 10, 0.000001)
+            self._finish_progress()
 
         else:
+            assert self._remaining_funcs is not None and self._func_queue is not None
             self._remaining_funcs.value = len(self._func_addrs)
 
             # generate a call tree (obviously, it's acyclic)
@@ -190,9 +240,7 @@ class CompleteCallingConventionsAnalysis(Analysis):
                         dependents[callee].add(func_addr)
 
             # enqueue all leaf functions
-            for func_addr in list(
-                k for k in depends_on if not depends_on[k]
-            ):  # pylint:disable=consider-using-dict-items
+            for func_addr in [k for k in depends_on if not depends_on[k]]:  # pylint:disable=consider-using-dict-items
                 self._func_queue.put((func_addr, None))
                 del depends_on[func_addr]
 
@@ -200,11 +248,17 @@ class CompleteCallingConventionsAnalysis(Analysis):
             cc_callback = self._cc_callback
             self._cc_callback = None
 
+            if self.project.simos is not None and isinstance(self.project.simos, SimWindows):
+                # delayed import
+                from angr.procedures.definitions import load_win32api_definitions
+
+                Initializer.get().register(load_win32api_definitions)
+
             # spawn workers to perform the analysis
             with self._func_queue_lock:
                 procs = [
-                    _mp_context.Process(target=self._worker_routine, args=(Initializer.get(),), daemon=True)
-                    for _ in range(self._workers)
+                    _mp_context.Process(target=self._worker_routine, args=(worker_id, Initializer.get()), daemon=True)
+                    for worker_id in range(self._workers)
                 ]
                 for proc_idx, proc in enumerate(procs):
                     self._update_progress(0, text=f"Spawning worker {proc_idx}...")
@@ -216,12 +270,20 @@ class CompleteCallingConventionsAnalysis(Analysis):
             self._update_progress(0)
             idx = 0
             while idx < total_funcs:
-                func_addr, cc, proto, varman = self._results.get(True)
+                try:
+                    with self._results_lock:
+                        func_addr, cc, proto, proto_libname, varman = self._results.get(True, timeout=0.01)
+                except queue.Empty:
+                    time.sleep(0.1)
+                    continue
+
                 func = self.kb.functions.get_by_addr(func_addr)
                 if cc is not None or proto is not None:
                     func.calling_convention = cc
-                    func.prototype = proto
-                    func.is_prototype_guessed = True
+                    self._set_function_prototype(func, proto, proto_libname)
+                    if proto_libname is not None:
+                        self.prototype_libnames.add(proto_libname)
+
                 if varman is not None:
                     self.kb.variables.function_managers[func_addr] = varman
                     varman.set_manager(self.kb.variables)
@@ -243,13 +305,16 @@ class CompleteCallingConventionsAnalysis(Analysis):
                         depends_on[dependent].discard(func_addr)
                         if not depends_on[dependent]:
                             callee_prototypes = self._get_callees_cc_prototypes(dependent)
-                            self._func_queue.put((dependent, callee_prototypes))
+                            with self._func_queue_lock:
+                                self._func_queue.put((dependent, callee_prototypes))
                             del depends_on[dependent]
 
             for proc in procs:
                 proc.join()
 
-    def _worker_routine(self, initializer: Initializer):
+            self._finish_progress()
+
+    def _worker_routine(self, worker_id: int, initializer: Initializer):
         initializer.initialize()
         idx = 0
         while self._remaining_funcs.value > 0:
@@ -262,32 +327,46 @@ class CompleteCallingConventionsAnalysis(Analysis):
                 continue
 
             if callee_info is not None:
-                callee_info: Dict[int, Tuple[Optional["SimCC"], Optional["SimTypeFunction"]]]
-                for callee, (callee_cc, callee_proto) in callee_info.items():
+                callee_info: dict[int, tuple[SimCC | None, SimTypeFunction | None, str | None]]
+                for callee, (callee_cc, callee_proto, callee_proto_libname) in callee_info.items():
                     callee_func = self.kb.functions.get_by_addr(callee)
                     callee_func.calling_convention = callee_cc
-                    callee_func.prototype = callee_proto
+                    self._set_function_prototype(callee_func, callee_proto, callee_proto_libname)
 
             idx += 1
-            if self._low_priority:
-                if idx % 3 == 0:
-                    time.sleep(0.1)
+            if self._low_priority and idx % 3 == 0:
+                time.sleep(0.1)
 
             try:
-                cc, proto, varman = self._analyze_core(func_addr)
+                cc, proto, proto_libname, varman = self._analyze_core(func_addr)
             except Exception:  # pylint:disable=broad-except
-                _l.error("Exception occurred during _analyze_core().", exc_info=True)
-                cc, proto, varman = None, None, None
-            self._results.put((func_addr, cc, proto, varman))
+                _l.error("Worker %d: Exception occurred during _analyze_core().", worker_id, exc_info=True)
+                cc, proto, proto_libname, varman = None, None, None, None
+            with self._results_lock:
+                self._results.put((func_addr, cc, proto, proto_libname, varman))
 
     def _analyze_core(
         self, func_addr: int
-    ) -> Tuple[Optional["SimCC"], Optional["SimTypeFunction"], Optional["VariableManagerInternal"]]:
+    ) -> tuple[SimCC | None, SimTypeFunction | None, str | None, VariableManagerInternal | None]:
         func = self.kb.functions.get_by_addr(func_addr)
         if func.ran_cca:
-            return func.calling_convention, func.prototype, self.kb.variables.get_function_manager(func_addr)
+            return (
+                func.calling_convention,
+                func.prototype,
+                func.prototype_libname,
+                self.kb.variables.get_function_manager(func_addr),
+            )
+
+        if (
+            self.mode == CallingConventionAnalysisMode.VARIABLES
+            and self._recover_variables
+            and self.function_needs_variable_recovery(func)
+        ):
+            # special case: we don't have a PCode-engine variable recovery analysis for PCode architectures!
+            if ":" in self.project.arch.name and self._func_graphs and func.addr in self._func_graphs:
+                # this is a pcode architecture
+                return None, None, None, None
 
-        if self._recover_variables and self.function_needs_variable_recovery(func):
             _l.info("Performing variable recovery on %r...", func)
             try:
                 _ = self.project.analyses[VariableRecoveryFast].prep(kb=self.kb)(
@@ -299,19 +378,29 @@ class CompleteCallingConventionsAnalysis(Analysis):
                     func.addr,
                     exc_info=True,
                 )
-                return None, None, None
+                return None, None, None, None
+
+        kwargs = {}
+        if self.mode == CallingConventionAnalysisMode.FAST:
+            facts = self.project.analyses[FactCollector].prep(kb=self.kb)(func)
+            kwargs["input_args"] = facts.input_args
+            kwargs["retval_size"] = facts.retval_size
 
         # determine the calling convention of each function
         cc_analysis = self.project.analyses[CallingConventionAnalysis].prep(kb=self.kb)(
-            func, cfg=self._cfg, analyze_callsites=self._analyze_callsites
+            func, cfg=self._cfg, analyze_callsites=self._analyze_callsites, **kwargs
         )
 
         if cc_analysis.cc is not None:
             _l.info("Determined calling convention and prototype for %r.", func)
-            return cc_analysis.cc, cc_analysis.prototype, self.kb.variables.get_function_manager(func_addr)
-        else:
-            _l.info("Cannot determine calling convention for %r.", func)
-            return None, None, self.kb.variables.get_function_manager(func_addr)
+            return (
+                cc_analysis.cc,
+                cc_analysis.prototype,
+                cc_analysis.prototype_libname if cc_analysis.prototype_libname is not None else func.prototype_libname,
+                self.kb.variables.get_function_manager(func_addr),
+            )
+        _l.info("Cannot determine calling convention for %r.", func)
+        return None, None, None, self.kb.variables.get_function_manager(func_addr)
 
     def prioritize_functions(self, func_addrs_to_prioritize: Iterable[int]):
         """
@@ -335,12 +424,12 @@ class CompleteCallingConventionsAnalysis(Analysis):
 
     def _get_callees_cc_prototypes(
         self, caller_func_addr: int
-    ) -> Dict[int, Tuple[Optional["SimCC"], Optional["SimTypeFunction"]]]:
+    ) -> dict[int, tuple[SimCC | None, SimTypeFunction | None, str | None]]:
         d = {}
         for callee in self.kb.functions.callgraph.successors(caller_func_addr):
             if callee != caller_func_addr and callee not in d:
                 func = self.kb.functions.get_by_addr(callee)
-                tpl = func.calling_convention, func.prototype
+                tpl = func.calling_convention, func.prototype, func.prototype_libname
                 d[callee] = tpl
         return d
 
@@ -365,10 +454,8 @@ class CompleteCallingConventionsAnalysis(Analysis):
         :rtype:         bool
         """
 
-        if func.is_simprocedure or func.is_plt:
-            return False
         # TODO: Check SimLibraries
-        return True
+        return not (func.is_simprocedure or func.is_plt)
 
 
 register_analysis(CompleteCallingConventionsAnalysis, "CompleteCallingConventions")
