@@ -1,3 +1,4 @@
+from __future__ import annotations
 from archinfo.arch_arm import is_arm_arch
 import cle
 
@@ -5,9 +6,8 @@ from .plugin import KnowledgeBasePlugin
 
 
 class Labels(KnowledgeBasePlugin):
-
     def __init__(self, kb):
-        self._kb = kb
+        super().__init__(kb=kb)
         self._labels = {}
         self._reverse_labels = {}
 
@@ -16,7 +16,14 @@ class Labels(KnowledgeBasePlugin):
             for v in obj.symbols:
                 if is_arm and v.name in {"$d", "$t", "$a"}:
                     continue
-                if v.name and not v.is_import and v.type not in {cle.SymbolType.TYPE_OTHER, }:
+                if (
+                    v.name
+                    and not v.is_import
+                    and v.type
+                    not in {
+                        cle.SymbolType.TYPE_OTHER,
+                    }
+                ):
                     self._labels[v.rebased_addr] = v.name
                     self._reverse_labels[v.name] = v.rebased_addr
             try:
@@ -77,8 +84,8 @@ class Labels(KnowledgeBasePlugin):
 
     def copy(self):
         o = Labels(self._kb)
-        o._labels = {k: v for k, v in self._labels.items()}
-        o._reverse_labels = {k: v for k, v in self._reverse_labels.items()}
+        o._labels = dict(self._labels.items())
+        o._reverse_labels = dict(self._reverse_labels.items())
 
     def get_unique_label(self, label):
         """
@@ -88,16 +95,16 @@ class Labels(KnowledgeBasePlugin):
         :return:            A unique label name.
         """
 
-        if label not in self._labels:
+        if label not in self._reverse_labels:
             return label
 
         # use it as the prefix
         i = 1
         while True:
-            new_label = "%s_%d" % (label, i)
-            if new_label not in self._labels:
+            new_label = f"{label}_{i}"
+            if new_label not in self._reverse_labels:
                 return new_label
             i += 1
 
 
-KnowledgeBasePlugin.register_default('labels', Labels)
+KnowledgeBasePlugin.register_default("labels", Labels)
