@@ -1,9 +1,6 @@
+from __future__ import annotations
 import angr
 
-######################################
-# __errno_location
-######################################
-
 
 class __errno_location(angr.SimProcedure):
     def run(self):  # pylint:disable=arguments-differ
