@@ -1,9 +1,6 @@
+from __future__ import annotations
 import angr
 
-######################################
-# xalloc_die
-######################################
-
 
 class xalloc_die(angr.SimProcedure):
     """
