@@ -1,9 +1,6 @@
+from __future__ import annotations
 import angr
 
-######################################
-# std::__throw_bad_alloc
-######################################
-
 
 class std____throw_bad_cast(angr.SimProcedure):  # pylint:disable=redefined-builtin
     # pylint:disable=arguments-differ
