@@ -1,10 +1,7 @@
+from __future__ import annotations
 import angr
 from angr.sim_type import SimTypeInt
 
-######################################
-# getuid
-######################################
-
 
 class getuid(angr.SimProcedure):
     # pylint: disable=arguments-differ
