@@ -1,8 +1,6 @@
+from __future__ import annotations
 import angr
 
-######################################
-# bind (but not really)
-######################################
 import logging
 
 l = logging.getLogger(name=__name__)
