@@ -1,10 +1,11 @@
 #!/usr/bin/env python3
 # pylint: disable=missing-class-docstring,no-self-use,line-too-long
+from __future__ import annotations
 
 import unittest
-from typing import Dict
 
 import archinfo
+import pydemumble
 
 import angr
 from angr.sim_type import (
@@ -21,7 +22,10 @@ from angr.sim_type import (
     SimTypeNum,
     SimTypeReference,
     SimTypeBottom,
+    SimTypeTop,
     SimTypeString,
+    dereference_simtype,
+    SimTypeCppFunction,
 )
 from angr.utils.library import convert_cproto_to_py, convert_cppproto_to_py
 
@@ -62,57 +66,99 @@ class TestTypes(unittest.TestCase):
     def test_cppproto_conversion(self):
         # a demangled class constructor prototype, without parameter names
         proto_0 = (
-            "std::basic_ifstream<char, std::char_traits<char>>::{ctor}(std::__cxx11::basic_string<char, "
-            "std::char_traits<char>, std::allocator<char>> const&, std::_Ios_Openmode)"
+            "std::basic_ifstream<char, std::char_traits<char>>::basic_ifstream<char, std::char_traits<char>>("
+            "std::__cxx11::basic_string<char, std::char_traits<char>, std::allocator<char>> const&, std::_Ios_Openmode)"
         )
         name, proto, _ = convert_cppproto_to_py(proto_0, with_param_names=False)
+        assert proto is not None
         assert proto.ctor is True
-        assert name == "std::basic_ifstream::__ctor__"
+        assert name == "std::basic_ifstream<char, std::char_traits<char>>::basic_ifstream<char, std::char_traits<char>>"
         assert len(proto.args) == 3
         assert isinstance(proto.args[0], SimTypePointer)  # this
         assert isinstance(proto.args[1], SimTypeReference)
         assert isinstance(proto.args[1].refs, SimTypeString)
-        assert proto.args[1].refs.name == "std::__cxx11::basic_string"
-        assert proto.args[1].refs.unqualified_name(lang="c++") == "basic_string"
+        assert (
+            proto.args[1].refs.name == "std::__cxx11::basic_string<char, std::char_traits<char>, std::allocator<char>>"
+        )
+        # assert proto.args[1].refs.unqualified_name(lang="c++") == "basic_string"
 
         proto_1 = "void std::basic_string<CharT,Traits,Allocator>::push_back(CharT ch)"
         name, proto, _ = convert_cppproto_to_py(proto_1, with_param_names=True)
-        assert name == "std::basic_string::push_back"
+        assert proto is not None
+        assert name == "std::basic_string<CharT, Traits, Allocator>::push_back"
         assert isinstance(proto.returnty, SimTypeBottom)
         assert isinstance(proto.args[0], SimTypePointer)  # this
         assert isinstance(proto.args[1], SimTypeChar)
 
         proto_2 = "void std::basic_string<CharT,Traits,Allocator>::swap(basic_string& other)"
         name, proto, _ = convert_cppproto_to_py(proto_2, with_param_names=True)
-        assert name == "std::basic_string::swap"
+        assert proto is not None
+        assert name == "std::basic_string<CharT, Traits, Allocator>::swap"
         assert isinstance(proto.returnty, SimTypeBottom)
         assert isinstance(proto.args[0], SimTypePointer)  # this
         assert isinstance(proto.args[1], SimTypeReference)
         assert isinstance(proto.args[1].refs, SimTypeString)
 
-        proto_3 = "std::ios_base::{base dtor}()"
+        proto_3 = "std::ios_base::~ios_base()"
         name, proto, _ = convert_cppproto_to_py(proto_3, with_param_names=True)
-        assert name == "std::ios_base::__base_dtor__"
+        assert proto is not None
+        assert name == "std::ios_base::~ios_base"
         assert proto.dtor is True
         assert isinstance(proto.returnty, SimTypeBottom)
 
-        proto_4 = "std::ios_base::{base dtor}()"
+        proto_4 = "std::ios_base::~ios_base()"
         name, proto, _ = convert_cppproto_to_py(proto_4, with_param_names=True)
-        assert name == "std::ios_base::__base_dtor__"
+        assert proto is not None
+        assert name == "std::ios_base::~ios_base"
 
         proto_5 = "void foo(int & bar);"
         name, proto, _ = convert_cppproto_to_py(proto_5, with_param_names=True)
+        assert proto is not None
         assert name == "foo"
         # note that there is no "this" pointer
+        assert len(proto.args) == 1
         assert isinstance(proto.args[0], SimTypeReference)
         assert isinstance(proto.args[0].refs, SimTypeInt)
         assert isinstance(proto.returnty, SimTypeBottom)
 
+    def test_cppproto_parse_operator_new(self):
+        proto = "operator new(unsigned long)"
+        _, proto, _ = convert_cppproto_to_py(proto)
+        assert proto is not None
+        assert len(proto.args) == 1
+
+    def test_cppproto_parse_operator_shl(self):
+        proto = "std::ostream::operator<<(std::ostream& (*)(std::ostream&))"
+        _, proto, _ = convert_cppproto_to_py(proto)
+        assert proto is not None
+        assert len(proto.args) == 2
+
+    def test_cppproto_parse_class_destructor(self):
+        mangled_proto = "??1?$basic_string@DU?$char_traits@D@std@@V?$allocator@D@2@@std@@QAE@XZ"
+        proto = pydemumble.demangle(mangled_proto)
+        _, proto, _ = convert_cppproto_to_py(proto)
+        assert isinstance(proto, SimTypeCppFunction)
+        assert proto.dtor is True
+        assert proto.convention == "__thiscall"
+        assert len(proto.args) == 1
+
+    def test_cppproto_parse_std_string_operator_equals(self):
+        mangled_proto = "??4?$basic_string@DU?$char_traits@D@std@@V?$allocator@D@2@@std@@QAEAAV01@ABV01@@Z"
+        proto = pydemumble.demangle(mangled_proto)
+        _, proto, _ = convert_cppproto_to_py(proto)
+        assert isinstance(proto, SimTypeCppFunction)
+        assert proto.ctor is False
+        assert proto.dtor is False
+        assert proto.convention == "__thiscall"
+        assert len(proto.args) == 2
+
     def test_struct_deduplication(self):
         angr.types.register_types(angr.types.parse_type("struct ahdr { int a ;}"))
         angr.types.register_types(angr.types.parse_type("struct bhdr { int b ;}"))
         angr.types.register_types(angr.types.parse_type("struct chdr { int c ;}"))
         dhdr = angr.types.parse_type("struct dhdr { struct ahdr a; struct bhdr b; struct chdr c;}")
+        assert isinstance(dhdr, SimStruct)
+        assert isinstance(dhdr.fields["a"], SimStruct)
         assert dhdr.fields["a"].fields
 
     def test_parse_type(self):
@@ -166,12 +212,16 @@ class TestTypes(unittest.TestCase):
 
     def test_self_referential_struct_or_union(self):
         struct_llist = angr.types.parse_type("struct llist { int data; struct llist *next; }")
+        assert isinstance(struct_llist, SimStruct)
+        assert isinstance(struct_llist.fields["next"], SimTypePointer)
         next_struct_llist = struct_llist.fields["next"].pts_to
         assert len(next_struct_llist.fields) == 2
         assert isinstance(next_struct_llist.fields["data"], SimTypeInt)
         assert isinstance(next_struct_llist.fields["next"], SimTypePointer)
 
         union_heap = angr.types.parse_type("union heap { int data; union heap *forward; }")
+        assert isinstance(union_heap, SimUnion)
+        assert isinstance(union_heap.members["forward"], SimTypePointer)
         forward_union_heap = union_heap.members["forward"].pts_to
         assert len(forward_union_heap.members) == 2
         assert isinstance(forward_union_heap.members["data"], SimTypeInt)
@@ -182,6 +232,8 @@ class TestTypes(unittest.TestCase):
         angr.types.register_types(angr.types.parse_type("struct b"))
         a = angr.types.parse_type("struct a { struct b *b_ptr; }")
         b = angr.types.parse_type("struct b { struct a *a_ptr; }")
+        assert isinstance(a, SimStruct)
+        assert isinstance(b, SimStruct)
 
         assert len(a.fields) == 1
         assert isinstance(a.fields["b_ptr"], SimTypePointer)
@@ -197,6 +249,8 @@ class TestTypes(unittest.TestCase):
         angr.types.register_types(angr.types.parse_type("union b"))
         a = angr.types.parse_type("union a { union b *b_ptr; }")
         b = angr.types.parse_type("union b { union a *a_ptr; }")
+        assert isinstance(a, SimUnion)
+        assert isinstance(b, SimUnion)
 
         assert len(a.members) == 1
         assert isinstance(a.members["b_ptr"], SimTypePointer)
@@ -209,16 +263,16 @@ class TestTypes(unittest.TestCase):
         assert b.members["a_ptr"].pts_to.name == "a"
 
     def test_top_type(self):
-        angr.types.register_types({"undefined": angr.types.SimTypeTop()})
-        fdef: Dict[str, SimTypeFunction] = angr.types.parse_defns("undefined f(undefined param_1, int param_2);")
+        angr.types.register_types({"undefined": SimTypeTop()})
+        fdef: dict[str, SimTypeFunction] = angr.types.parse_defns("undefined f(undefined param_1, int param_2);")
         sig = fdef["f"]
-        assert sig.args == [angr.types.SimTypeTop(), angr.types.SimTypeInt()]
+        assert sig.args == (SimTypeTop(), SimTypeInt())
 
     def test_arg_names(self):
-        angr.types.register_types({"undefined": angr.types.SimTypeTop()})
-        fdef: Dict[str, SimTypeFunction] = angr.types.parse_defns("int f(int param_1, int param_2);")
+        angr.types.register_types({"undefined": SimTypeTop()})
+        fdef: dict[str, SimTypeFunction] = angr.types.parse_defns("int f(int param_1, int param_2);")
         sig = fdef["f"]
-        assert sig.arg_names == ["param_1", "param_2"]
+        assert sig.arg_names == ("param_1", "param_2")
 
         # Check that arg_names survive a with_arch call
         nsig = sig.with_arch(archinfo.ArchAMD64())
@@ -228,11 +282,11 @@ class TestTypes(unittest.TestCase):
 
         # If for some reason only some of the parameters are named,
         # the list can only be partially not None, but has to match the positions
-        fdef: Dict[str, SimTypeFunction] = angr.types.parse_defns("int f(int param1, int);")
+        fdef: dict[str, SimTypeFunction] = angr.types.parse_defns("int f(int param1, int);")
         sig = fdef["f"]
-        assert sig.arg_names == ["param1", None]
+        assert sig.arg_names == ("param1", None)
 
-        fdef: Dict[str, SimTypeFunction] = angr.types.parse_defns("int f();")
+        fdef: dict[str, SimTypeFunction] = angr.types.parse_defns("int f();")
         sig = fdef["f"]
         assert sig.arg_names == ()
 
@@ -248,6 +302,8 @@ class TestTypes(unittest.TestCase):
 
     def test_forward_declaration_typedef_struct(self):
         _, extra_types = angr.types.parse_file("typedef struct _A A; struct _A {int a;int b;};")
+        assert isinstance(extra_types["A"], SimStruct)
+        assert isinstance(extra_types["struct _A"], SimStruct)
 
         assert extra_types["A"].fields is not None
         assert isinstance(extra_types["A"].fields["a"], SimTypeInt)
@@ -259,6 +315,8 @@ class TestTypes(unittest.TestCase):
 
     def test_forward_declaration_typedef_union(self):
         _, extra_types = angr.types.parse_file("typedef union _A A; union _A {int a;int b;};")
+        assert isinstance(extra_types["A"], SimUnion)
+        assert isinstance(extra_types["union _A"], SimUnion)
 
         assert extra_types["A"].members is not None
         assert isinstance(extra_types["A"].members["a"], SimTypeInt)
@@ -280,6 +338,7 @@ class TestTypes(unittest.TestCase):
             char*       name;
         }"""
         ty = angr.types.parse_type(code)
+        assert isinstance(ty, SimStruct)
         ty = ty.with_arch(archinfo.ArchAArch64())
         assert [(t.size, t.offset) for t in list(ty.fields.values())[1:-1]] == [
             (36, 0),
@@ -289,6 +348,24 @@ class TestTypes(unittest.TestCase):
             (1, 7),
         ]
 
+    def test_dereference_type_anonymous_struct(self):
+        angr.procedures.definitions.load_win32_type_collections()
+        variant_type = angr.SIM_TYPE_COLLECTIONS["win32"].get("VARIANT")
+        assert isinstance(variant_type, SimStruct)
+        assert isinstance(variant_type.fields["Anonymous"], SimUnion)
+        assert variant_type.fields["Anonymous"].members["Anonymous"].anonymous is True
+        t = dereference_simtype(variant_type, [angr.SIM_TYPE_COLLECTIONS["win32"]]).with_arch(archinfo.ArchX86())
+        assert t.size > 0  # an exception is raised if anonymous structs are not handled correctly
+
+    def test_simunion_size_bottom_types(self):
+        union_type = SimUnion(
+            {"filterType": SimTypeBottom(label="Guid"), "calloutKey": SimTypeBottom(label="Guid")},
+            name="<anon>",
+            label="None",
+        )
+        union_type = union_type.with_arch(archinfo.ArchAMD64())
+        assert union_type.size == 8  # fall back to architecture word size
+
 
 class TestSimTypeFunction(unittest.TestCase):
     def test_c_repr(self):
